package sk.tuke.gamestudio.game.mines.webui;


import sk.tuke.gamestudio.entity.Score;
import sk.tuke.gamestudio.game.mines.core.Field;
import sk.tuke.gamestudio.game.mines.core.GameState;
import sk.tuke.gamestudio.game.mines.core.TileState;
import sk.tuke.gamestudio.service.ScoreService;

import java.util.Date;

public class WebUI {

    private final String GAME_NAME = "mines";
    private Field field;
    private boolean gameEnded = false;
    private int finalScore = 0;

    private ScoreService scoreService;

    public WebUI(ScoreService scoreService) {
        this.scoreService = scoreService;
        createField();
    }

    /**
     * Implements what should happen when a command comes.
     */
    public void processCommand(Command command, String rowString, String columnString) {
        if (command == Command.NEW) {
            createField();
            return;
        }

        if (gameEnded) {
            return;
        }

        try {
            int row = Integer.parseInt(rowString);
            int column = Integer.parseInt(columnString);

            switch (command) {
                case OPEN:
                    field.openTile(row, column);
                    break;
                case MARK:
                    if (field.getTile(row, column).getState() == TileState.CLOSED) {
                        field.markTile(row, column);
                    }
                    break;
                case UNMARK:
                    if (field.getTile(row, column).getState() == TileState.MARKED) {
                        field.markTile(row, column);
                    }
                    break;
            }
        } catch (NumberFormatException e) {
            System.out.println("WebUI: Incorrect command parameters for 'processInput()'.\n" + e.getMessage());
        }

        if(isGameEnded() && !gameEnded) {
            gameEnded = true;
            if(isGameWon()) {
                finalScore = field.getScore();
                scoreService.addScore(
                        // TODO: request username from currently playing user,
                        //  since because the game will be running on server, system username will be always the same!
                        new Score(GAME_NAME, System.getProperty("user.name"), finalScore, new Date()));
            }
            return;
        }

    }

    private void createField() {
        gameEnded = false;
        finalScore = 0;
        field = new Field(9, 9, 10);
    }

    public Field getField() {
        return field;
    }

    public ViewTile<?> getViewTile(int row, int column) {
        return new ViewTile<>(field.getTile(row, column), row, column);
    }

    public boolean isGameWon() {
        return field.getState() == GameState.SOLVED;
    }

    public boolean isGameLost() {
        return field.getState() == GameState.FAILED;
    }

    public boolean isGameEnded() {
        return isGameLost() || isGameWon();
    }

    public int getFinalScore() {
        return finalScore;
    }

}