package sk.tuke.gamestudio.server.controller;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.context.WebApplicationContext;
import sk.tuke.gamestudio.entity.Score;
import sk.tuke.gamestudio.game.mines.webui.Command;
import sk.tuke.gamestudio.game.mines.webui.WebUI;
import sk.tuke.gamestudio.service.ScoreService;

import java.util.List;

@Controller
@Scope(WebApplicationContext.SCOPE_SESSION)
public class MinesController {

    private final WebUI webUI;

    private final ScoreService scoreService;

    public MinesController(ScoreService scoreService) {
        this.scoreService = scoreService;
        this.webUI = new WebUI(scoreService);
    }

    @RequestMapping("/mines")
    public String mines(@RequestParam(value = "command", required = false) String command,
                        @RequestParam(value = "row", required = false) String row,
                        @RequestParam(value = "column", required = false) String column,
                        Model model) {
        Command cmd;
        if (command == null) {
            cmd = ((row != null) && (column != null)) ? Command.OPEN : Command.NEW;
        } else {
            cmd = Command.valueOf(command.toUpperCase());
        }

        webUI.processCommand(cmd, row, column);
        model.addAttribute("webUI", webUI);
        List<Score> bestScores = scoreService.getTopScores("mines");
        model.addAttribute("scores", bestScores);

        return "mines"; //same name as the template

    }
}
